unit CiaListBox;

{  Version information:

   1.01 1 feb 2003
      - Added Version property
      - Added property AcceptExtern, where it can drop items from other
        TCiaListBox
   1.02 2 feb 2003
      - Added property AcceptDelete to autodelete if item is dropped from other
        TCiaListBox
      - Added protection if source has no items    }

interface

uses
  Classes, Controls, StdCtrls;

const
   CiaListBox_Version = '1.02';

type
  TChangeOrderNotifyEvent = procedure(Sender : TObject; FromIndex, ToIndex : integer) of object;

  TCiaListBox = class(TListBox)
  private
    FVersion: string;
    FAcceptExtern: boolean;
    FAcceptDelete: boolean;
    FOnChangeItemOrder : TChangeOrderNotifyEvent;
    procedure SetVersion(Value: string);
  protected
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure DragDrop(Source: TObject; X, Y: Integer); override;
  published
    property OnChangeItemOrder: TChangeOrderNotifyEvent read FOnChangeItemOrder write FOnChangeItemOrder;
    property AcceptExtern: boolean read FAcceptExtern write FAcceptExtern;
    property AcceptDelete: boolean read FAcceptDelete write FAcceptDelete;
    property Version: string read FVersion write SetVersion;
  end;

procedure Register;

implementation

constructor TCiaListBox.Create(AOwner: TComponent);
begin
   inherited;
   DragMode := dmAutomatic;
   FVersion := CiaListBox_Version;
end;

procedure TCiaListBox.DragDrop(Source: TObject; X, Y: Integer);
var
   ToIndex: integer;
   S: TListBox;
begin
   inherited;
   ToIndex := ItemAtPos(Point(x, y), True);
   if Assigned(FOnChangeItemOrder) then begin
      if ToIndex = -1 then
         ToIndex := Items.Count - 1;
      if ItemIndex <> ToIndex then
         FOnChangeItemOrder(self, ItemIndex, ToIndex);
   end;
   if Source = Self then
      Items.Move(ItemIndex, ToIndex)
   else begin
      S := TListBox(Source);
      Items.Insert(ToIndex, S.Items[S.ItemIndex]);
      if FAcceptDelete then
         S.Items.Delete(S.ItemIndex);
   end;
   ItemIndex := ToIndex;

   {ToIndex := ItemAtPos(Point(x, y), True);
   if Assigned(FOnChangeItemOrder) then begin
      if ToIndex = -1 then
         ToIndex := Items.Count - 1;
      if ItemIndex <> ToIndex then
         FOnChangeItemOrder(self, ItemIndex, ToIndex);
   end;
   Items.Move(ItemIndex, ToIndex);
   ItemIndex := ToIndex; }
end;

procedure TCiaListBox.DragOver(Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
   inherited;
   if FAcceptExtern then
      Accept := Source is TCiaListBox
   else
      Accept := Source = Self;
   Accept := TListBox(Source).Items.Count > 0;
end;

procedure TCiaListBox.SetVersion(Value: string);
begin
end;

procedure Register;
begin
  RegisterComponents('Cia', [TCiaListBox]);
end;

end.
